export declare type Maybe<T> = T | undefined;
export interface GenSequence<T> extends IterableIterator<T> {
    /** map values from type T to type U */
    map<U>(fnMap: (t: T) => U): GenSequence<U>;
    /** keep values where the fnFilter(t) returns true */
    filter(fnFilter: (t: T) => boolean): GenSequence<T>;
    /** reduce function see Array.reduce */
    reduce(fnReduce: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue?: T): Maybe<T>;
    reduce<U>(fnReduce: (previousValue: U, currentValue: T, currentIndex: number, array: T[]) => U, initialValue: U): Maybe<U>;
    scan(fnReduce: (previousValue: T, currentValue: T, currentIndex: number, array: T[]) => T, initialValue?: T): GenSequence<T>;
    scan<U>(fnReduce: (previousValue: U, currentValue: T, currentIndex: number, array: T[]) => U, initialValue: U): GenSequence<U>;
    combine<U, V>(fn: (t: T, u?: U) => V, j: Iterable<U>): GenSequence<V>;
    concat(j: Iterable<T>): GenSequence<T>;
    concatMap<U>(fn: (t: T) => Iterable<U>): GenSequence<U>;
    skip(n: number): GenSequence<T>;
    take(n: number): GenSequence<T>;
    first(fnFilter?: (t: T) => boolean): Maybe<T>;
    toArray(): T[];
    toIterable(): IterableIterator<T>;
}
export interface GenIterable<T> {
    [Symbol.iterator](): IterableIterator<T>;
}
export declare function GenSequence<T>(i: GenIterable<T>): GenSequence<T>;
export declare function filter<T>(fnFilter: (t: T) => boolean, i: Iterable<T>): IterableIterator<T>;
export declare function reduce<T, U>(fnReduce: (prevValue: U, curValue: T, curIndex: number) => U, initialValue: U, i: Iterable<T>): Maybe<U>;
export declare function scan<T, U>(i: Iterable<T>, fnReduce: (prevValue: U, curValue: T, curIndex: number) => U, initValue: U): IterableIterator<U>;
/**
 * apply a mapping function to an Iterable.
 */
export declare function map<T, U>(fnMap: (t: T) => U): (i: Iterable<T>) => IterableIterator<U>;
export declare function map<T, U>(fnMap: (t: T) => U, i: Iterable<T>): IterableIterator<U>;
/**
 * Combine two iterables together using fnMap function.
 */
export declare function combine<T, U, V>(fnMap: (t: T, u?: U) => V, i: Iterable<T>, j: Iterable<U>): IterableIterator<V>;
/**
 * Concat two iterables together
 */
export declare function concat<T>(i: Iterable<T>, j: Iterable<T>): IterableIterator<T>;
/**
 * Creates a scan function that can be used in a map function.
 */
export declare function scanMap<T, U>(accFn: (acc: U, value: T) => U, init: U): ((value: T) => U);
export declare function skip<T>(n: number, i: Iterable<T>): IterableIterator<T>;
export declare function take<T>(n: number, i: Iterable<T>): IterableIterator<T>;
export declare function concatMap<T, U>(fn: (t: T) => Iterable<U>, i: Iterable<T>): IterableIterator<U>;
export declare function first<T>(fn: Maybe<(t: T) => boolean>, i: Iterable<T>): Maybe<T>;
export declare function iterate<T>(i: Iterable<T>): IterableIterator<T>;
export default GenSequence;
